//
//  PhotoEditingViewController.swift
//  Grayscale
//
//  Created by Jake Marsh on 8/11/15.
//  Copyright © 2015 Magnus. All rights reserved.
//

import UIKit
import Photos
import PhotosUI
import CoreImage
import OpenGLES

class PhotoEditingViewController: UIViewController, PHContentEditingController {
  var input: PHContentEditingInput?
  var output: PHContentEditingOutput?

  var processing = false {
    didSet {
      processingLabel.hidden = !processing
      imageView.alpha = processing ? 0.7 : 1.0
    }
  }

  @IBOutlet weak var processingLabel: UILabel!

  @IBOutlet weak var imageView: UIImageView!

  func blackAndWhite(image: CIImage) -> CIImage {
    var image = CIFilter(name: "CIColorControls", withInputParameters: [
      kCIInputImageKey : image,
      "inputBrightness" : 0.0,
      "inputContrast" : 1.1,
      "inputSaturation" : 0.0
    ])!.outputImage

    image = CIFilter(name: "CIExposureAdjust", withInputParameters: [
      kCIInputImageKey : image,
      "inputEV" : 0.7
    ])!.outputImage

    return image
  }

  func grayscaleOutputFromInput() -> PHContentEditingOutput? {
    guard let input = input else { return nil }
    guard let url = input.fullSizeImageURL else { return nil }

    let orientation = input.fullSizeImageOrientation

    guard var inputImage = CIImage(contentsOfURL: url, options: nil) else { return nil }
    inputImage = inputImage.imageByApplyingOrientation(orientation)

    let bwImage = blackAndWhite(inputImage)

    let data = jpegRepresentationOfImage(bwImage)

    let output = PHContentEditingOutput(contentEditingInput: input)

    data.writeToURL(output.renderedContentURL, atomically: true)

    output.adjustmentData = PHAdjustmentData(
      formatIdentifier: NSBundle.mainBundle().bundleIdentifier!,
      formatVersion: "1.0",
      data: "Grayscale".dataUsingEncoding(NSUTF8StringEncoding, allowLossyConversion: false)!
    )

    return output
  }

  func jpegRepresentationOfImage(image: CIImage) -> NSData {
    let eaglContext = EAGLContext(API: .OpenGLES2)
    let ciContext = CIContext(EAGLContext: eaglContext)
    
    let outputImageRef = ciContext.createCGImage(image, fromRect: image.extent)
    let uiImage = UIImage(CGImage: outputImageRef, scale: 1.0, orientation: UIImageOrientation.Up)

    return UIImageJPEGRepresentation(uiImage, 0.9)! //this takes upwards of 20-30 seconds with large photos!
  }

  @IBAction func processImage(sender: UIButton) {
    guard !processing else { return }

    processing = true

    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0)) {
      self.output = self.grayscaleOutputFromInput()

      guard let output = self.output else { return }

      dispatch_async(dispatch_get_main_queue()) {
        let data: NSData?

        do {
          data = try NSData(contentsOfURL: output.renderedContentURL, options: .DataReadingMappedIfSafe)
        } catch _ {
          data = nil
        }

        self.imageView.image = UIImage(data: data!)
        self.processing = false
      }
    }
  }

  // MARK: - PHContentEditingController

  func canHandleAdjustmentData(adjustmentData: PHAdjustmentData?) -> Bool {
    return false
  }

  func startContentEditingWithInput(contentEditingInput: PHContentEditingInput?, placeholderImage: UIImage) {
    input = contentEditingInput

    imageView.image = placeholderImage
  }

  func finishContentEditingWithCompletionHandler(completionHandler: ((PHContentEditingOutput!) -> Void)!) {
    completionHandler?(output)
  }

  var shouldShowCancelConfirmation: Bool {
    // Determines whether a confirmation to discard changes should be shown to the user on cancel.
    // (Typically, this should be "true" if there are any unsaved changes.)
    return false
  }

  func cancelContentEditing() {
      // Clean up temporary files, etc.
      // May be called after finishContentEditingWithCompletionHandler: while you prepare output.
  }
}
